%   Description: Takes a matrix poloynomial, a, and computes the eigenvalues,
%   eigenvectors and backward error by using tropical scaling.
%   Input: 
    %   a =[A_0,A_1,...,A_n]: a matrix polynomial
%   Outputs: 
    %   eigns: vector of eigenvalues.
    %   eignv: matrix of eigenvectors. eignv(:,i) corresponds to the
    %   eigns(i).
    %   bkerr: vector of backward errors. bkerr(i) corresponds to the
    %   backward error for an eigen pair, eigns(i) & eignv(:,i).
function [eigns,eignvs,bkerr]=tropicalscaling(a)    
    ind=0;
    n=size(a,1);
    if n<2
        error('The coefficients should be matrices not scalars.');
    end
    pdeg=(size(a,2)/n)-1;
    if pdeg<2
        error('The tropical scaling requires the degree to be greater than one.');
    end
    eigns=zeros(n*pdeg,1);
    bkerr=zeros(n*pdeg,1);
    scaleda=zeros(n,(pdeg+1)*n);    
    eigns=zeros(n*pdeg,1);
    p=zeros(1,pdeg+1);
    %computing the tropical roots    
    for i=1: pdeg+1
        tmp=a(1:n,1+(i-1)*n:(i*n));
        p(pdeg-i+2)=norm(tmp,2);
    end;  
    trplst=tropical_roots(p,0);  
    trplen=size(trplst,1);
    trplst=[trplst,zeros(trplen,1)];
    disp(sprintf('\n'));
    disp('          ****************************************');
    disp('          Tropical root       Multiplicity');   
    disp(sprintf('\n'));
    for i=1:trplen        
        troot=trplst(i,2);
        tmp1=[];
        tmp1=[size(p,2)-1:-1:0];
        tmp1=(troot.^tmp1).*p;
        trplst(i,3)=max(tmp1);        
        disp(sprintf('          %e          %d       ',troot,trplst(i,1)));
    end 
    disp('          ****************************************');
    disp(sprintf('\n'));
    if (size(find(isnan(trplst)),1)>0) | (size(find(isinf(trplst)),1)>0)
        error('at least one of the tropical roots or the value of tropical polynomial is infinite or NAN')
    end
    %  
    for i=1:trplen
        %computing scaled matrix polynomial
        troot=trplst(trplen-i+1,2);
        mul_troot=trplst(trplen-i+1,1);
        scalingalphai=1/trplst(trplen-i+1,3);
        for j=0:pdeg
            scaleda(:,j*n+1:(j+1)*n)=scalingalphai*(troot^j)*a(:,j*n+1:(j+1)*n);
        end; 
        %
        %computing companion form
        A=diag(ones((pdeg-1)*n,1),n);
        A(n*(pdeg-1)+1:n*pdeg,:)=-scaleda(:,1:pdeg*n);
        E=eye(n*pdeg,n*pdeg);
        E(n*(pdeg-1)+1:n*pdeg,n*(pdeg-1)+1:n*pdeg)=scaleda(:,pdeg*n+1:(pdeg+1)*n);
        %      
        %computing the eigenvalues and the eigenvectors using qz algorithm
        [AA,BB,Q,Z,V] = qz(A,E);
        tmp1=diag(AA);
        tmp2=diag(BB);
        num_inf=0;               
        if size(find(tmp2==0),1)>0
            warning('QZ produces some INF eigenvalues');  
            tt=find(tmp2==0);
            V(:,tt)=[];
            tmp2(tt)=[];
            tmp1(tt)=[];
            eignlst=tmp1./tmp2;
            num_inf=size(tt,1);
        else
            eignlst=tmp1./tmp2;
        end
        tmp1=[];
        tmp2=[];
        [tmp2,tmp1]=sort(abs(eignlst),'descend');
        eignlst= eignlst*troot;
        if (ind*n+1)<num_inf
            error('The computation of a group of eigenvalues failed');
        else
            eigns(ind*n+1:(ind+mul_troot)*n)=eignlst(tmp1(ind*n+1-num_inf:(ind+mul_troot)*n-num_inf));
            eignvs(1:n*pdeg,ind*n+1:(ind+mul_troot)*n)=V(:,tmp1(ind*n+1-num_inf:(ind+mul_troot)*n-num_inf)); 
        end
        ind=ind+mul_troot;
    end;
    %computing backwarderror
    dim=n*pdeg;    
    tmp1=[];
    tmp2=[];      
    for i=1:dim      
        eigenvector=eignvs(1:n,i);
        eigenvalue=eigns(i);
        tmp1=zeros(n,n);         
        for j=1: pdeg+1
            tmp2=a(1:n,1+(j-1)*n:(j*n));
            tmp1=tmp1+eigenvalue^(j-1)*tmp2;
        end;          
        r=norm(tmp1*eigenvector,2);       
        tmp1=[];
        tmp1=[size(p,2)-1:-1:0];
        tmp1=(abs(eigenvalue).^tmp1).*p;
        alpha=sum(tmp1);    
        if (alpha==0) | (norm(eigenvector,2)==0) 
            warning('The backward error is infinite for some eigenvalue');
            bkerr(i)=inf;
        else
            bkerr(i)=r/(alpha*norm(eigenvector,2));
        end;        
    end;    
end
